"""
    Mixin class for Common methods.
    
    If you want to create new rows or column types you just subclass this 
    class. Then make sure it has this field::
    
        StringField(
            name='macroName',
            default_method='getDefaultMacro',
            widget=StringWidget(
                label='Rowmacro',
            )
        ),
    
    That should be all that is needed.
"""

from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View
from Products.Archetypes.atapi import DisplayList

class Common:
    
    security = ClassSecurityInfo()

    exclude_from_nav = True
    
    # Actually defined in the mixin classes
    MACRO_TEMPLATES = ''
    DEFAULT_MACRO = ''
    
    # redirects to the pagebuilder, so we don't see the row and col views after base_edit
    
    security.declareProtected(View, 'pagebuilder_redirect')
    def pagebuilder_redirect(self):
        "Redirects to the absolute url of the pagebuilder"
        return self.REQUEST.RESPONSE.redirect(self.pagebuilder_url())

    # macro selection mechanism

    security.declarePrivate('getMacros')
    def getMacros(self):
        "Returns the possible macros"
        return getattr(self, self.MACRO_TEMPLATES).macros

    security.declareProtected(View, 'getMacro')
    def getMacro(self):
        "Returns the selected macro"
        macros = self.getMacros()
        return macros.get(self.getMacroName(), macros[self.getDefaultMacro()])

    security.declareProtected(View, 'getMacroNames')
    def getMacroNames(self):
        "Returns the macronames sorted by sortval (value after last _)"
        names = self.getMacros().keys()
        decorated = []
        for name in names:
            sortval = name.split('_')[-1:]
            decorated.append( (sortval, name) )
        decorated.sort()
        return [d[-1] for d in decorated]

    security.declareProtected(View, 'getDefaultMacro')
    def getDefaultMacro(self):
        "Returns the default macro. Is the first after sort."
        return self.getMacroNames()[0]
        

    security.declareProtected(View, 'getMacrosItems')
    def getMacrosItems(self):
        "Returns list [(id, title),]"
        macrosMap = []
        for key in self.getMacroNames():
            try:
                # looks for this pattern: name_SORTVAL
                # and returns 'name' part that has been 'Titled'
                # XXX: this method sucks and should be changed. I really
                # wanted to use a 'title' attribute in the macro,
                # but getting that from the macro got needlessly convoluted
                title = ' '.join(key.split('_')[:-1]).title()
            except:
                title = key
            macrosMap.append( (key, title) )
        return macrosMap

    security.declareProtected(View, 'getMacrosDisplayList')
    def getMacrosDisplayList(self):
        "Returns list DisplayList([(id, title),])"
        return DisplayList(self.getMacrosItems())

